==> genstudio-2025.3.8.dev202503101208.tar.gz: genstudio-2025.3.8.dev202503101208/src/genstudio/js/components/bitmap.tsx <==
import * as React from 'react';
import { useContext, useEffect, useMemo, useRef } from 'react';
import { $StateContext } from '../context';
import { useContainerWidth } from '../utils';

/**
 * Props interface for the bitmap component
 */
interface BitmapProps {
    /** Raw pixel data as Uint8Array/Uint8ClampedArray */
    pixels: Uint8Array | Uint8ClampedArray;
    /** Width of the bitmap in pixels */
    width: number;
    /** Height of the bitmap in pixels */
    height: number;
}

/**
 * Renders raw pixel data as a bitmap image in a canvas element.
 *
 * Supports both RGB (3 bytes per pixel) and RGBA (4 bytes per pixel) formats.
 * RGB data is automatically converted to RGBA by setting alpha to 255.
 * The canvas is scaled to fit the container width while maintaining aspect ratio.
 *
 * @param pixels - Raw pixel data as Uint8Array/Uint8ClampedArray
 * @param width - Width of the bitmap in pixels
 * @param height - Height of the bitmap in pixels
 * @returns React component rendering the bitmap
 */
export function Bitmap({pixels, width, height}: BitmapProps) {
    const canvasRef = useRef<HTMLCanvasElement | null>(null);
    const [ref, containerWidth] = useContainerWidth();
    const $state = useContext($StateContext);
    const done = useMemo(() => $state.beginUpdate("bitmap"), [])

    useEffect(() => {
        const canvas = canvasRef.current;
        if (!canvas) return;

        const ctx = canvas.getContext('2d');
        if (!ctx) return;

        const bytesPerPixel = pixels.length / (width * height);

        // Create ImageData based on pixel format
        let imageData: ImageData;
        if (bytesPerPixel === 3) {
            // Convert RGB to RGBA
            const rgba = new Uint8ClampedArray(width * height * 4);
            for (let i = 0; i < pixels.length; i += 3) {
                const j = (i / 3) * 4;
                rgba[j] = pixels[i];
                rgba[j + 1] = pixels[i + 1];
                rgba[j + 2] = pixels[i + 2];
                rgba[j + 3] = 255;
            }
            imageData = new ImageData(rgba, width, height);
        } else {
            // Assume RGBA
            imageData = new ImageData(new Uint8ClampedArray(pixels), width, height);
        }

        ctx.putImageData(imageData, 0, 0);
        done();
    }, [pixels, width, height]);

    return <div ref={ref}>
        <canvas
            ref={canvasRef}
            width={width}
            height={height}
            style={{
                width: containerWidth,
            }}
        />
    </div>;
}

==> genstudio-2025.3.9.dev202503130834.tar.gz: genstudio-2025.3.9.dev202503130834/src/genstudio/js/components/bitmap.tsx <==
import * as React from 'react';
import { useContext, useEffect, useMemo, useRef } from 'react';
import { $StateContext } from '../context';
import { useContainerWidth } from '../utils';

/**
 * Props interface for the bitmap component
 */
interface BitmapProps {
    /** Raw pixel data as Uint8Array/Uint8ClampedArray */
    pixels: Uint8Array | Uint8ClampedArray;
    /** Width of the bitmap in pixels */
    width: number;
    /** Height of the bitmap in pixels */
    height: number;
    /** CSS styles to apply to the canvas element */
    style?: React.CSSProperties;
    /** How to interpolate pixels when scaling */
    interpolation?: 'nearest' | 'bilinear';
}

/**
 * Renders raw pixel data as a bitmap image in a canvas element.
 *
 * Supports both RGB (3 bytes per pixel) and RGBA (4 bytes per pixel) formats.
 * RGB data is automatically converted to RGBA by setting alpha to 255.
 * The canvas is scaled to fit the container width while maintaining aspect ratio.
 *
 * @param pixels - Raw pixel data as Uint8Array/Uint8ClampedArray
 * @param width - Width of the bitmap in pixels
 * @param height - Height of the bitmap in pixels
 * @param interpolation - How to interpolate pixels when scaling
 * @returns React component rendering the bitmap
 */
export function Bitmap({pixels, width, height, interpolation = 'nearest', style, ...props}: BitmapProps) {
    const [ref, containerWidth] = useContainerWidth();
    const $state = useContext($StateContext);
    const done = useMemo(() => $state.beginUpdate("bitmap"), [containerWidth])

    useEffect(() => {
        const container = ref.current;
        if (!container) {
            console.warn('Container ref not available');
            return;
        }

        // Remove any existing canvas elements
        while (container.firstChild) {
            container.removeChild(container.firstChild);
        }

        // Create new canvas
        const canvas = document.createElement('canvas');
        canvas.width = width;
        canvas.height = height;
        canvas.style.width = `${containerWidth}px`;
        canvas.style.imageRendering = interpolation === 'nearest' ? 'pixelated' : 'auto';
        Object.assign(canvas.style, style);

        const ctx = canvas.getContext('2d');
        if (!ctx) {
            console.warn('Could not get 2d context');
            return;
        }

        const bytesPerPixel = pixels.length / (width * height);

        // Create ImageData based on pixel format
        let imageData: ImageData;
        if (bytesPerPixel === 3) {
            // Convert RGB to RGBA
            const rgba = new Uint8ClampedArray(width * height * 4);
            for (let i = 0; i < pixels.length; i += 3) {
                const j = (i / 3) * 4;
                rgba[j] = pixels[i];
                rgba[j + 1] = pixels[i + 1];
                rgba[j + 2] = pixels[i + 2];
                rgba[j + 3] = 255;
            }
            imageData = new ImageData(rgba, width, height);
        } else {
            // Assume RGBA
            imageData = new ImageData(new Uint8ClampedArray(pixels), width, height);
        }

        ctx.putImageData(imageData, 0, 0);
        container.appendChild(canvas);
        done();

    }, [pixels, width, height, interpolation, containerWidth, style, ref]);

    return <div ref={ref} {...props} />;
}
